@@ -2,7 +2,7 @@
  *
  *  AT chat library with GLib integration
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -27,11 +27,12 @@
 
 #include "gatsyntax.h"
 
-enum GSMV1_STATE_ {
+enum GSMV1_STATE {
 	GSMV1_STATE_IDLE = 0,
 	GSMV1_STATE_INITIAL_CR,
 	GSMV1_STATE_INITIAL_LF,
 	GSMV1_STATE_RESPONSE,
+	GSMV1_STATE_RESPONSE_STRING,
 	GSMV1_STATE_TERMINATOR_CR,
 	GSMV1_STATE_GUESS_MULTILINE_RESPONSE,
 	GSMV1_STATE_MULTILINE_RESPONSE,
@@ -41,8 +42,17 @@ enum GSMV1_STATE_ {
 	GSMV1_STATE_PDU,
 	GSMV1_STATE_PDU_CR,
 	GSMV1_STATE_PROMPT,
-	GSMV1_STATE_GARBAGE,
-	GSMV1_STATE_GARBAGE_CHECK_LF,
+	GSMV1_STATE_ECHO,
+	GSMV1_PPP_DATA,
+};
+
+enum GSM_PERMISSIVE_STATE {
+	GSM_PERMISSIVE_STATE_IDLE = 0,
+	GSM_PERMISSIVE_STATE_RESPONSE,
+	GSM_PERMISSIVE_STATE_RESPONSE_STRING,
+	GSM_PERMISSIVE_STATE_GUESS_PDU,
+	GSM_PERMISSIVE_STATE_PDU,
+	GSM_PERMISSIVE_STATE_PROMPT,
 };
 
 static void gsmv1_hint(GAtSyntax *syntax, GAtSyntaxExpectHint hint)
@@ -72,15 +82,17 @@ static GAtSyntaxResult gsmv1_feed(GAtSyntax *syntax,
 		case GSMV1_STATE_IDLE:
 			if (byte == '\r')
 				syntax->state = GSMV1_STATE_INITIAL_CR;
+			else if (byte == '~')
+				syntax->state = GSMV1_PPP_DATA;
 			else
-				syntax->state = GSMV1_STATE_GARBAGE;
+				syntax->state = GSMV1_STATE_ECHO;
 			break;
 
 		case GSMV1_STATE_INITIAL_CR:
 			if (byte == '\n')
 				syntax->state = GSMV1_STATE_INITIAL_LF;
 			else
-				syntax->state = GSMV1_STATE_GARBAGE;
+				syntax->state = GSMV1_STATE_ECHO;
 			break;
 
 		case GSMV1_STATE_INITIAL_LF:
@@ -88,6 +100,8 @@ static GAtSyntaxResult gsmv1_feed(GAtSyntax *syntax,
 				syntax->state = GSMV1_STATE_TERMINATOR_CR;
 			else if (byte == '>')
 				syntax->state = GSMV1_STATE_PROMPT;
+			else if (byte == '"')
+				syntax->state = GSMV1_STATE_RESPONSE_STRING;
 			else
 				syntax->state = GSMV1_STATE_RESPONSE;
 			break;
@@ -95,6 +109,13 @@ static GAtSyntaxResult gsmv1_feed(GAtSyntax *syntax,
 		case GSMV1_STATE_RESPONSE:
 			if (byte == '\r')
 				syntax->state = GSMV1_STATE_TERMINATOR_CR;
+			else if (byte == '"')
+				syntax->state = GSMV1_STATE_RESPONSE_STRING;
+			break;
+
+		case GSMV1_STATE_RESPONSE_STRING:
+			if (byte == '"')
+				syntax->state = GSMV1_STATE_RESPONSE;
 			break;
 
 		case GSMV1_STATE_TERMINATOR_CR:
@@ -179,13 +200,11 @@ static GAtSyntaxResult gsmv1_feed(GAtSyntax *syntax,
 			syntax->state = GSMV1_STATE_RESPONSE;
 			return G_AT_SYNTAX_RESULT_UNSURE;
 
-		case GSMV1_STATE_GARBAGE:
-			if (byte == '\r')
-				syntax->state = GSMV1_STATE_GARBAGE_CHECK_LF;
+		case GSMV1_STATE_ECHO:
 			/* This handles the case of echo of the PDU terminated
 			 * by CtrlZ character
 			 */
-			else if (byte == 26) {
+			if (byte == 26 || byte == '\r') {
 				syntax->state = GSMV1_STATE_IDLE;
 				res = G_AT_SYNTAX_RESULT_UNRECOGNIZED;
 				i += 1;
@@ -194,14 +213,95 @@ static GAtSyntaxResult gsmv1_feed(GAtSyntax *syntax,
 
 			break;
 
-		case GSMV1_STATE_GARBAGE_CHECK_LF:
-			syntax->state = GSMV1_STATE_IDLE;
-			res = G_AT_SYNTAX_RESULT_UNRECOGNIZED;
-
-			if (byte == '\n')
+		case GSMV1_PPP_DATA:
+			if (byte == '~') {
+				syntax->state = GSMV1_STATE_IDLE;
+				res = G_AT_SYNTAX_RESULT_UNRECOGNIZED;
 				i += 1;
+				goto out;
+			}
 
-			goto out;
+			break;
+
+		default:
+			break;
+		};
+
+		i += 1;
+	}
+
+out:
+	*len = i;
+	return res;
+}
+
+static void gsm_permissive_hint(GAtSyntax *syntax, GAtSyntaxExpectHint hint)
+{
+	if (hint == G_AT_SYNTAX_EXPECT_PDU)
+		syntax->state = GSM_PERMISSIVE_STATE_GUESS_PDU;
+}
+
+static GAtSyntaxResult gsm_permissive_feed(GAtSyntax *syntax,
+						const char *bytes, gsize *len)
+{
+	gsize i = 0;
+	GAtSyntaxResult res = G_AT_SYNTAX_RESULT_UNSURE;
+
+	while (i < *len) {
+		char byte = bytes[i];
+
+		switch (syntax->state) {
+		case GSM_PERMISSIVE_STATE_IDLE:
+			if (byte == '\r' || byte == '\n')
+				/* ignore */;
+			else if (byte == '>')
+				syntax->state = GSM_PERMISSIVE_STATE_PROMPT;
+			else
+				syntax->state = GSM_PERMISSIVE_STATE_RESPONSE;
+			break;
+
+		case GSM_PERMISSIVE_STATE_RESPONSE:
+			if (byte == '\r') {
+				syntax->state = GSM_PERMISSIVE_STATE_IDLE;
+
+				i += 1;
+				res = G_AT_SYNTAX_RESULT_LINE;
+				goto out;
+			} else if (byte == '"')
+				syntax->state =
+					GSM_PERMISSIVE_STATE_RESPONSE_STRING;
+			break;
+
+		case GSM_PERMISSIVE_STATE_RESPONSE_STRING:
+			if (byte == '"')
+				syntax->state = GSM_PERMISSIVE_STATE_RESPONSE;
+			break;
+
+		case GSM_PERMISSIVE_STATE_GUESS_PDU:
+			if (byte != '\r' && byte != '\n')
+				syntax->state = GSM_PERMISSIVE_STATE_PDU;
+			break;
+
+		case GSM_PERMISSIVE_STATE_PDU:
+			if (byte == '\r') {
+				syntax->state = GSM_PERMISSIVE_STATE_IDLE;
+
+				i += 1;
+				res = G_AT_SYNTAX_RESULT_PDU;
+				goto out;
+			}
+			break;
+
+		case GSM_PERMISSIVE_STATE_PROMPT:
+			if (byte == ' ') {
+				syntax->state = GSM_PERMISSIVE_STATE_IDLE;
+				i += 1;
+				res = G_AT_SYNTAX_RESULT_PROMPT;
+				goto out;
+			}
+
+			syntax->state = GSM_PERMISSIVE_STATE_RESPONSE;
+			return G_AT_SYNTAX_RESULT_UNSURE;
 
 		default:
 			break;
@@ -232,11 +332,17 @@ GAtSyntax *g_at_syntax_new_full(GAtSyntaxFeedFunc feed,
 }
 
 
-GAtSyntax *g_at_syntax_new_gsmv1()
+GAtSyntax *g_at_syntax_new_gsmv1(void)
 {
 	return g_at_syntax_new_full(gsmv1_feed, gsmv1_hint, GSMV1_STATE_IDLE);
 }
 
+GAtSyntax *g_at_syntax_new_gsm_permissive(void)
+{
+	return g_at_syntax_new_full(gsm_permissive_feed, gsm_permissive_hint,
+					GSM_PERMISSIVE_STATE_IDLE);
+}
+
 GAtSyntax *g_at_syntax_ref(GAtSyntax *syntax)
 {
 	if (syntax == NULL)
