@@ -57,6 +57,7 @@
 #include <ofono/radio-settings.h>
 #include <ofono/audio-settings.h>
 #include <ofono/stk.h>
+#include <ofono/ctm.h>
 #include <ofono/log.h>
 
 #include <drivers/atmodem/atutil.h>
@@ -433,17 +434,10 @@ error:
 static void setup_internal_mux(struct ofono_modem *modem)
 {
 	struct ifx_data *data = ofono_modem_get_data(modem);
-	GIOFlags flags;
 	int i;
 
 	DBG("");
 
-	flags = g_io_channel_get_flags(data->device) | G_IO_FLAG_NONBLOCK;
-	g_io_channel_set_flags(data->device, flags, NULL);
-
-	g_io_channel_set_encoding(data->device, NULL, NULL);
-	g_io_channel_set_buffered(data->device, FALSE);
-
 	data->mux = g_at_mux_new_gsm0710_basic(data->device, data->frame_size);
 	if (data->mux == NULL)
 		goto error;
@@ -667,14 +661,10 @@ static void ifx_set_online(struct ofono_modem *modem, ofono_bool_t online,
 
 	DBG("%p %s", modem, online ? "online" : "offline");
 
-	if (cbd == NULL)
-		goto error;
-
 	if (g_at_chat_send(data->dlcs[AUX_DLC], command, NULL,
 					set_online_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, cbd->data);
@@ -692,6 +682,7 @@ static void ifx_pre_sim(struct ofono_modem *modem)
 	ofono_voicecall_create(modem, 0, "ifxmodem", data->dlcs[VOICE_DLC]);
 	ofono_audio_settings_create(modem, 0,
 					"ifxmodem", data->dlcs[VOICE_DLC]);
+	ofono_ctm_create(modem, 0, "ifxmodem", data->dlcs[AUX_DLC]);
 }
 
 static void ifx_post_sim(struct ofono_modem *modem)
@@ -705,6 +696,9 @@ static void ifx_post_sim(struct ofono_modem *modem)
 					"atmodem", data->dlcs[AUX_DLC]);
 	ofono_call_forwarding_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_radio_settings_create(modem, 0, "ifxmodem", data->dlcs[AUX_DLC]);
+
+	ofono_sms_create(modem, OFONO_VENDOR_IFX,
+					"atmodem", data->dlcs[AUX_DLC]);
 }
 
 static void ifx_post_online(struct ofono_modem *modem)
@@ -719,8 +713,6 @@ static void ifx_post_online(struct ofono_modem *modem)
 	ofono_netreg_create(modem, OFONO_VENDOR_IFX,
 					"atmodem", data->dlcs[NETREG_DLC]);
 
-	ofono_sms_create(modem, OFONO_VENDOR_IFX,
-					"atmodem", data->dlcs[AUX_DLC]);
 	ofono_cbs_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 	ofono_ussd_create(modem, 0, "atmodem", data->dlcs[AUX_DLC]);
 
