@@ -234,7 +234,7 @@ struct error_entry ceer_errors[] = {
 	{ 127,	"Interworking, unspecified" },
 };
 
-gboolean valid_phone_number_format(const char *number)
+gboolean valid_number_format(const char *number, int length)
 {
 	int len = strlen(number);
 	int begin = 0;
@@ -246,7 +246,7 @@ gboolean valid_phone_number_format(const char *number)
 	if (number[0] == '+')
 		begin = 1;
 
-	if ((len - begin) > OFONO_MAX_PHONE_NUMBER_LENGTH)
+	if ((len - begin) > length)
 		return FALSE;
 
 	for (i = begin; i < len; i++) {
@@ -262,6 +262,22 @@ gboolean valid_phone_number_format(const char *number)
 	return TRUE;
 }
 
+/*
+ * According to 3GPP TS 24.011 or 3GPP TS 31.102, some
+ * addresses (or numbers), like Service Centre address,
+ * Destination address, or EFADN (Abbreviated dialling numbers),
+ * are up 20 digits.
+ */
+gboolean valid_phone_number_format(const char *number)
+{
+	return valid_number_format(number, 20);
+}
+
+gboolean valid_long_phone_number_format(const char *number)
+{
+	return valid_number_format(number, OFONO_MAX_PHONE_NUMBER_LENGTH);
+}
+
 gboolean valid_cdma_phone_number_format(const char *number)
 {
 	int len = strlen(number);
@@ -714,29 +730,6 @@ const char *registration_tech_to_string(int tech)
 	}
 }
 
-const char *packet_bearer_to_string(int bearer)
-{
-	switch (bearer) {
-	case PACKET_BEARER_NONE:
-		return "none";
-	case PACKET_BEARER_GPRS:
-		return "gprs";
-	case PACKET_BEARER_EGPRS:
-		return "edge";
-	case PACKET_BEARER_UMTS:
-		return "umts";
-	case PACKET_BEARER_HSUPA:
-		return "hsupa";
-	case PACKET_BEARER_HSDPA:
-		return "hsdpa";
-	case PACKET_BEARER_HSUPA_HSDPA:
-		return "hspa";
-	case PACKET_BEARER_EPS:
-		return "lte";
-	}
-	return "";
-}
-
 gboolean is_valid_apn(const char *apn)
 {
 	int i;
@@ -769,3 +762,10 @@ const char *ofono_uuid_to_str(const struct ofono_uuid *uuid)
 
 	return encode_hex_own_buf(uuid->uuid, OFONO_SHA1_UUID_LEN, 0, buf);
 }
+
+void ofono_call_init(struct ofono_call *call)
+{
+	memset(call, 0, sizeof(struct ofono_call));
+	call->cnap_validity = CNAP_VALIDITY_NOT_AVAILABLE;
+	call->clip_validity = CLIP_VALIDITY_NOT_AVAILABLE;
+}
